# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, overload

import httpx

from ..types import completion_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._streaming import Stream, AsyncStream
from .._base_client import make_request_options
from ..types.completion_chunk import CompletionChunk
from ..types.completion_create_response import CompletionCreateResponse

__all__ = ["CompletionsResource", "AsyncCompletionsResource"]


class CompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return CompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return CompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stream: Optional[Literal[False]] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        stream: Literal[True],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[CompletionChunk]:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        stream: bool,
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | Stream[CompletionChunk]:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt"], ["model", "prompt", "stream"])
    def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stream: Optional[Literal[False]] | Literal[True] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | Stream[CompletionChunk]:
        return self._post(
            "/v1/completions"
            if self._client._base_url_overridden
            else "https://api.fireworks.ai/inference/v1/completions",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "context_length_exceeded_behavior": context_length_exceeded_behavior,
                    "echo": echo,
                    "echo_last": echo_last,
                    "frequency_penalty": frequency_penalty,
                    "ignore_eos": ignore_eos,
                    "images": images,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "min_p": min_p,
                    "mirostat_lr": mirostat_lr,
                    "mirostat_target": mirostat_target,
                    "n": n,
                    "perf_metrics_in_response": perf_metrics_in_response,
                    "prediction": prediction,
                    "presence_penalty": presence_penalty,
                    "prompt_cache_isolation_key": prompt_cache_isolation_key,
                    "prompt_cache_max_len": prompt_cache_max_len,
                    "raw_output": raw_output,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "response_format": response_format,
                    "return_token_ids": return_token_ids,
                    "seed": seed,
                    "speculation": speculation,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "typical_p": typical_p,
                    "user": user,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionCreateResponse,
            stream=stream or False,
            stream_cls=Stream[CompletionChunk],
        )


class AsyncCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stream: Optional[Literal[False]] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        stream: Literal[True],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[CompletionChunk]:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        stream: bool,
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | AsyncStream[CompletionChunk]:
        """
        Create a completion for the provided prompt and parameters.

        Args:
          model: The name of the model to use.

              Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`

          prompt: The prompt to generate completions for.

              It can be a single string or an array of strings.

              It can also be an array of integers or an array of integer arrays, which allows
              to pass already tokenized prompt.

              If multiple prompts are specified, several choices with corresponding `index`
              will be returned in the output.

          stream: Whether to stream back partial progress. If set, tokens will be sent as
              data-only
              [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
              as they become available, with the stream terminated by a `data: [DONE]`
              message.

          context_length_exceeded_behavior: What to do if the token count of prompt plus `max_tokens` exceeds the model's
              context window.

              Passing `truncate` limits the `max_tokens` to at most
              `context_window_length - prompt_length`. This is the default.

              Passing `error` would trigger a request error.

              The default of `'truncate'` is selected as it allows to ask for high
              `max_tokens` value while respecting the context window length without having to
              do client-side prompt tokenization.

              Note, that it differs from OpenAI's behavior that matches that of `error`.

          echo: Echo back the prompt in addition to the completion.

          echo_last: Echo back the last N tokens of the prompt in addition to the completion. This is
              useful for obtaining logprobs of the prompt suffix but without transferring too
              much data. Passing `echo_last=len(prompt)` is the same as `echo=True`

          frequency_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on their
              existing frequency in the text so far, decreasing the model's likelihood to
              repeat the same line verbatim.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `presence_penalty` for penalizing tokens that have at least one
              appearance at a fixed rate.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          ignore_eos: This setting controls whether the model should ignore the End of Sequence (EOS)
              token. When set to `True`, the model will continue generating tokens even after
              the EOS token is produced. By default, it stops when the EOS token is reached.

          images: The list of base64 encoded images for visual language completition generation.

              They should be formatted as MIME_TYPE,<base64 encoded str>

              eg. data:image/jpeg;base64,<base64 encoded str>

              Additionally, the number of images provided should match the number of '<image>'
              special token in the prompt

          logit_bias: Modify the likelihood of specified tokens appearing in the completion. Accepts a
              json object that maps tokens (specified by their token ID in the tokenizer) to
              an associated bias value from -100 to 100. Mathematically, the bias is added to
              the logits generated by the model prior to sampling.

          logprobs: Include log probabilities in the response. This accepts either a boolean or an
              integer:

              If set to `true`, log probabilities are included and the number of alternatives
              can be controlled via `top_logprobs` (OpenAI-compatible behavior).

              If set to an integer N (0-5), include log probabilities for up to N most likely
              tokens per position in the legacy format.

              The API will always return the logprob of the sampled token, so there may be up
              to `logprobs+1` elements in the response when an integer is used. The maximum
              value for the integer form is 5.

          max_completion_tokens: Alias for max_tokens. Cannot be specified together with max_tokens.

          max_tokens: The maximum number of tokens to generate in the completion. If the token count
              of your prompt plus max_tokens exceeds the model's context length, the behavior
              depends on context_length_exceeded_behavior. By default, max_tokens will be
              lowered to fit in the context window instead of returning an error.

          metadata: Additional metadata to store with the request for tracing/distillation.

          min_p: Minimum probability threshold for token selection. Only tokens with
              probability >= min_p are considered for selection. This is an alternative to
              `top_p` and `top_k` sampling.

              Required range: `0 <= x <= 1`

          mirostat_lr: Specifies the learning rate for the Mirostat sampling algorithm, which controls
              how quickly the model adjusts its token distribution to maintain the target
              perplexity. A smaller value slows down the adjustments, leading to more stable
              but gradual shifts, while higher values speed up corrections at the cost of
              potential instability.

          mirostat_target: Defines the target perplexity for the Mirostat algorithm. Perplexity measures
              the unpredictability of the generated text, with higher values encouraging more
              diverse and creative outputs, while lower values prioritize predictability and
              coherence. The algorithm dynamically adjusts the token selection to maintain
              this target during text generation.

              If not specified, Mirostat sampling is disabled.

          n: How many completions to generate for each prompt.

              **Note:** Because this parameter generates many completions, it can quickly
              consume your token quota. Use carefully and ensure that you have reasonable
              settings for `max_tokens` and `stop`.

              Required range: `1 <= x <= 128`

              Example: `1`

          perf_metrics_in_response: Whether to include performance metrics in the response body.

              **Non-streaming requests:** Performance metrics are always included in response
              headers (e.g., `fireworks-prompt-tokens`,
              `fireworks-server-time-to-first-token`). Setting this to `true` additionally
              includes the same metrics in the response body under the `perf_metrics` field.

              **Streaming requests:** Performance metrics are only included in the response
              body under the `perf_metrics` field in the final chunk (when `finish_reason` is
              set). This is because headers may not be accessible during streaming.

              The response body `perf_metrics` field contains the following metrics:

              **Basic Metrics (all deployments):**

              - `prompt-tokens`: Number of tokens in the prompt
              - `server-time-to-first-token`: Time from request start to first token (in
                seconds)
              - `server-processing-time`: Total processing time (in seconds, only for
                completed requests)

              **Predicted Outputs Metrics:**

              - `speculation-prompt-tokens`: Number of speculative prompt tokens
              - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
                tokens (for completed requests)

              **Dedicated Deployment Only Metrics:**

              - `speculation-generated-tokens`: Number of speculative generated tokens (for
                completed requests)
              - `speculation-acceptance`: Speculation acceptance rates by position
              - `cached-prompt-tokens`: Number of cached prompt tokens
              - `backend-host`: Hostname of the backend server
              - `num-concurrent-requests`: Number of concurrent requests
              - `deployment`: Deployment name
              - `tokenizer-queue-duration`: Time spent in tokenizer queue
              - `tokenizer-duration`: Time spent in tokenizer
              - `prefill-queue-duration`: Time spent in prefill queue
              - `prefill-duration`: Time spent in prefill
              - `generation-queue-duration`: Time spent in generation queue

          prediction: OpenAI-compatible predicted output for speculative decoding. Can be a
              PredictedOutput object or a simple string. Automatically transformed to
              speculation.

          presence_penalty: Number between -2.0 and 2.0. Positive values penalize new tokens based on
              whether they appear in the text so far, increasing the model's likelihood to
              talk about new topics.

              Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
              samples somewhat. If the aim is to strongly suppress repetition, then one can
              increase the coefficients up to 2, but this can noticeably degrade the quality
              of samples. Negative values can be used to increase the likelihood of
              repetition.

              See also `frequency_penalty` for penalizing tokens at an increasing rate
              depending on how often they appear.

              OpenAI compatible (follows OpenAI's conventions for handling token frequency and
              repetition penalties).

              Required range: `-2 <= x <= 2`

          prompt_cache_isolation_key: Isolation key for prompt caching to separate cache entries.

          prompt_cache_max_len: Maximum length of the prompt to cache.

          raw_output: Return raw output from the model.

          reasoning_effort: Applicable to reasoning models only, this option controls the reasoning token
              length, and can be set to either `'none'`, `'low'`, `'medium'`, `'high'` or an
              integer. `'low'`, `'medium'` and `'high'` correspond to progressively higher
              thinking effort and thus longer reasoning tokens. `'none'` means disable
              thinking. You can alternatively set the option to an integer controlling the
              hard-cutoff for reasoning token length (this is not entirely OpenAI compatible,
              you might have to use fireworks.ai client library to bypass the schema check).
              Note: For OpenAI GPT OSS models, only the string values (`'low'`, `'medium'`,
              `'high'`) are supported. Integer values will not work with these models.

          repetition_penalty: Applies a penalty to repeated tokens to discourage or encourage repetition. A
              value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
              penalize repetition, reducing the likelihood of repeating tokens. Values between
              `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
              a good balance, a value of `1.2` is often recommended. Note that the penalty is
              applied to both the generated output and the prompt in decoder-only models.

              Required range: `0 <= x <= 2`

          response_format: Allows to force the model to produce specific output format.

              Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
              message the model generates is valid JSON.

              If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
              `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

              Important: when using JSON mode, it's crucial to also instruct the model to
              produce JSON via a system or user message. Without this, the model may generate
              an unending stream of whitespace until the generation reaches the token limit,
              resulting in a long-running and seemingly "stuck" request.

              Also note that the message content may be partially cut off if
              `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
              or the conversation exceeded the max context length. In this case the return
              value might not be a valid JSON.

          return_token_ids: Return token IDs alongside text to avoid retokenization drift.

          seed: Random seed for deterministic sampling.

          speculation: Speculative decoding prompt or token IDs to speed up generation.

          stop: Up to 4 sequences where the API will stop generating further tokens. The
              returned text will NOT contain the stop sequence.

          temperature: What sampling temperature to use, between 0 and 2. Higher values like 0.8 will
              make the output more random, while lower values like 0.2 will make it more
              focused and deterministic.

              We generally recommend altering this or top_p but not both.

              Required range: `0 <= x <= 2`

              Example: `1`

          top_k: Top-k sampling is another sampling method where the k most probable next tokens
              are filtered and the probability mass is redistributed among only those k next
              tokens. The value of k controls the number of candidates for the next token at
              each step during text generation. Must be between 0 and 100.

              Required range: `0 <= x <= 100`

              Example: `50`

          top_logprobs: An integer between 0 and 5 specifying the number of most likely tokens to return
              at each token position, each with an associated log probability. The minimum
              value is 0 and the maximum value is 5.

              When `logprobs` is set, `top_logprobs` can be used to modify how many top log
              probabilities are returned. If `top_logprobs` is not set, the API will return up
              to `logprobs` tokens per position.

              Required range: `0 <= x <= 5`

          top_p: An alternative to sampling with temperature, called nucleus sampling, where the
              model considers the results of the tokens with top_p probability mass. So 0.1
              means only the tokens comprising the top 10% probability mass are considered.

              We generally recommend altering this or temperature but not both.

              Required range: `0 <= x <= 1`

              Example: `1`

          typical_p: Typical-p sampling is an alternative to nucleus sampling. It considers the most
              typical tokens whose cumulative probability is at most typical_p.

              Required range: `0 <= x <= 1`

          user: A unique identifier representing your end-user, which can help monitor and
              detect abuse.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt"], ["model", "prompt", "stream"])
    async def create(
        self,
        *,
        model: str,
        prompt: Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]],
        context_length_exceeded_behavior: Literal["error", "truncate"] | Omit = omit,
        echo: Optional[bool] | Omit = omit,
        echo_last: Optional[int] | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        images: Union[SequenceNotStr[str], Iterable[SequenceNotStr[str]], None] | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logprobs: Union[int, bool, None] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        mirostat_lr: Optional[float] | Omit = omit,
        mirostat_target: Optional[float] | Omit = omit,
        n: int | Omit = omit,
        perf_metrics_in_response: Optional[bool] | Omit = omit,
        prediction: Optional[completion_create_params.Prediction] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        prompt_cache_isolation_key: Optional[str] | Omit = omit,
        prompt_cache_max_len: Optional[int] | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, None] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        response_format: Optional[completion_create_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        speculation: Union[str, Iterable[int], None] | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stream: Optional[Literal[False]] | Literal[True] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        typical_p: Optional[float] | Omit = omit,
        user: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompletionCreateResponse | AsyncStream[CompletionChunk]:
        return await self._post(
            "/v1/completions"
            if self._client._base_url_overridden
            else "https://api.fireworks.ai/inference/v1/completions",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "context_length_exceeded_behavior": context_length_exceeded_behavior,
                    "echo": echo,
                    "echo_last": echo_last,
                    "frequency_penalty": frequency_penalty,
                    "ignore_eos": ignore_eos,
                    "images": images,
                    "logit_bias": logit_bias,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "min_p": min_p,
                    "mirostat_lr": mirostat_lr,
                    "mirostat_target": mirostat_target,
                    "n": n,
                    "perf_metrics_in_response": perf_metrics_in_response,
                    "prediction": prediction,
                    "presence_penalty": presence_penalty,
                    "prompt_cache_isolation_key": prompt_cache_isolation_key,
                    "prompt_cache_max_len": prompt_cache_max_len,
                    "raw_output": raw_output,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "response_format": response_format,
                    "return_token_ids": return_token_ids,
                    "seed": seed,
                    "speculation": speculation,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "typical_p": typical_p,
                    "user": user,
                },
                completion_create_params.CompletionCreateParamsStreaming
                if stream
                else completion_create_params.CompletionCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompletionCreateResponse,
            stream=stream or False,
            stream_cls=AsyncStream[CompletionChunk],
        )


class CompletionsResourceWithRawResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_raw_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithRawResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_raw_response_wrapper(
            completions.create,
        )


class CompletionsResourceWithStreamingResponse:
    def __init__(self, completions: CompletionsResource) -> None:
        self._completions = completions

        self.create = to_streamed_response_wrapper(
            completions.create,
        )


class AsyncCompletionsResourceWithStreamingResponse:
    def __init__(self, completions: AsyncCompletionsResource) -> None:
        self._completions = completions

        self.create = async_to_streamed_response_wrapper(
            completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UserCreateParams"]


class UserCreateParams(TypedDict, total=False):
    role: Required[str]
    """The user's role, e.g. admin or user."""

    user_id: Annotated[str, PropertyInfo(alias="userId")]
    """The user ID to use in the user name.

    e.g. my-user If not specified, a default ID is generated from user.email.
    """

    display_name: Annotated[str, PropertyInfo(alias="displayName")]
    """Human-readable display name of the user.

    e.g. "Alice" Must be fewer than 64 characters long.
    """

    email: str
    """The user's email address."""

    service_account: Annotated[bool, PropertyInfo(alias="serviceAccount")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["InferenceParameters"]


class InferenceParameters(BaseModel):
    extra_body: Optional[str] = FieldInfo(alias="extraBody", default=None)
    """
    Additional parameters for the inference request as a JSON string. For example:
    "{\"stop\": [\"\\n\"]}".
    """

    max_tokens: Optional[int] = FieldInfo(alias="maxTokens", default=None)
    """Maximum number of tokens to generate per response."""

    n: Optional[int] = None
    """Number of response candidates to generate per input."""

    temperature: Optional[float] = None
    """Sampling temperature, typically between 0 and 2."""

    top_k: Optional[int] = FieldInfo(alias="topK", default=None)
    """Top-k sampling parameter, limits the token selection to the top k tokens."""

    top_p: Optional[float] = FieldInfo(alias="topP", default=None)
    """Top-p sampling parameter, typically between 0 and 1."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import APIKey, api_key_list_params, api_key_create_params, api_key_delete_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.api_key import APIKey
from ..types.api_key_param import APIKeyParam
from ..types.api_key_list_response import APIKeyListResponse

__all__ = ["APIKeysResource", "AsyncAPIKeysResource"]


class APIKeysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> APIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return APIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> APIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return APIKeysResourceWithStreamingResponse(self)

    def create(
        self,
        user_id: str,
        *,
        account_id: str,
        api_key: APIKeyParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKey:
        """
        Create API Key

        Args:
          api_key: The API key to be created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys",
            body=maybe_transform({"api_key": api_key}, api_key_create_params.APIKeyCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKey,
        )

    def list(
        self,
        user_id: str,
        *,
        account_id: str,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKeyListResponse:
        """
        List API Keys

        Args:
          filter: Field for filtering results.

          order_by: Field for ordering results.

          page_size: Number of API keys to return in the response. Pagination support to be added.

          page_token: Token for fetching the next page of results. Pagination support to be added.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    api_key_list_params.APIKeyListParams,
                ),
            ),
            cast_to=APIKeyListResponse,
        )

    def delete(
        self,
        user_id: str,
        *,
        account_id: str,
        key_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete API Key

        Args:
          key_id: The key ID for the API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys:delete"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys:delete",
            body=maybe_transform({"key_id": key_id}, api_key_delete_params.APIKeyDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAPIKeysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAPIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAPIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAPIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncAPIKeysResourceWithStreamingResponse(self)

    async def create(
        self,
        user_id: str,
        *,
        account_id: str,
        api_key: APIKeyParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKey:
        """
        Create API Key

        Args:
          api_key: The API key to be created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys",
            body=await async_maybe_transform({"api_key": api_key}, api_key_create_params.APIKeyCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKey,
        )

    async def list(
        self,
        user_id: str,
        *,
        account_id: str,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKeyListResponse:
        """
        List API Keys

        Args:
          filter: Field for filtering results.

          order_by: Field for ordering results.

          page_size: Number of API keys to return in the response. Pagination support to be added.

          page_token: Token for fetching the next page of results. Pagination support to be added.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    api_key_list_params.APIKeyListParams,
                ),
            ),
            cast_to=APIKeyListResponse,
        )

    async def delete(
        self,
        user_id: str,
        *,
        account_id: str,
        key_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete API Key

        Args:
          key_id: The key ID for the API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/users/{user_id}/apiKeys:delete"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}/apiKeys:delete",
            body=await async_maybe_transform({"key_id": key_id}, api_key_delete_params.APIKeyDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class APIKeysResourceWithRawResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = to_raw_response_wrapper(
            api_keys.create,
        )
        self.list = to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = to_raw_response_wrapper(
            api_keys.delete,
        )


class AsyncAPIKeysResourceWithRawResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = async_to_raw_response_wrapper(
            api_keys.create,
        )
        self.list = async_to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = async_to_raw_response_wrapper(
            api_keys.delete,
        )


class APIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = to_streamed_response_wrapper(
            api_keys.create,
        )
        self.list = to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = to_streamed_response_wrapper(
            api_keys.delete,
        )


class AsyncAPIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.create = async_to_streamed_response_wrapper(
            api_keys.create,
        )
        self.list = async_to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            api_keys.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.wandb_config import WandbConfig
from .shared_params.training_config import TrainingConfig

__all__ = ["DpoJobCreateParams"]


class DpoJobCreateParams(TypedDict, total=False):
    dataset: Required[str]
    """The name of the dataset used for training."""

    dpo_job_id: Annotated[str, PropertyInfo(alias="dpoJobId")]
    """ID of the DPO job, a random ID will be generated if not specified."""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    training_config: Annotated[TrainingConfig, PropertyInfo(alias="trainingConfig")]
    """Common training configurations."""

    wandb_config: Annotated[WandbConfig, PropertyInfo(alias="wandbConfig")]
    """The Weights & Biases team/user account for logging job progress."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .auto_tune import AutoTune
from .placement import Placement
from .shared.status import Status
from .autoscaling_policy import AutoscalingPolicy

__all__ = ["Deployment"]


class Deployment(BaseModel):
    base_model: str = FieldInfo(alias="baseModel")

    accelerator_count: Optional[int] = FieldInfo(alias="acceleratorCount", default=None)
    """
    The number of accelerators used per replica. If not specified, the default is
    the estimated minimum required by the base model.
    """

    accelerator_type: Optional[
        Literal[
            "ACCELERATOR_TYPE_UNSPECIFIED",
            "NVIDIA_A100_80GB",
            "NVIDIA_H100_80GB",
            "AMD_MI300X_192GB",
            "NVIDIA_A10G_24GB",
            "NVIDIA_A100_40GB",
            "NVIDIA_L4_24GB",
            "NVIDIA_H200_141GB",
            "NVIDIA_B200_180GB",
            "AMD_MI325X_256GB",
        ]
    ] = FieldInfo(alias="acceleratorType", default=None)
    """The type of accelerator to use."""

    active_model_version: Optional[str] = FieldInfo(alias="activeModelVersion", default=None)
    """
    The model version that is currently active and applied to running replicas of a
    deployment.
    """

    autoscaling_policy: Optional[AutoscalingPolicy] = FieldInfo(alias="autoscalingPolicy", default=None)

    auto_tune: Optional[AutoTune] = FieldInfo(alias="autoTune", default=None)
    """The performance profile to use for this deployment."""

    cluster: Optional[str] = None
    """If set, this deployment is deployed to a cloud-premise cluster."""

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the deployment."""

    delete_time: Optional[datetime] = FieldInfo(alias="deleteTime", default=None)
    """The time at which the resource will be soft deleted."""

    deployment_shape: Optional[str] = FieldInfo(alias="deploymentShape", default=None)
    """
    The name of the deployment shape that this deployment is using. On the server
    side, this will be replaced with the deployment shape version name.
    """

    deployment_template: Optional[str] = FieldInfo(alias="deploymentTemplate", default=None)
    """The name of the deployment template to use for this deployment.

    Only available to enterprise accounts.
    """

    description: Optional[str] = None
    """Description of the deployment."""

    desired_replica_count: Optional[int] = FieldInfo(alias="desiredReplicaCount", default=None)
    """The desired number of replicas for this deployment.

    This represents the target replica count that the system is trying to achieve.
    """

    direct_route_api_keys: Optional[List[str]] = FieldInfo(alias="directRouteApiKeys", default=None)
    """The set of API keys used to access the direct route deployment.

    If direct routing is not enabled, this field is unused.
    """

    direct_route_handle: Optional[str] = FieldInfo(alias="directRouteHandle", default=None)
    """The handle for calling a direct route.

    The meaning of the handle depends on the direct route type of the deployment:
    INTERNET -> The host name for accessing the deployment
    GCP_PRIVATE_SERVICE_CONNECT -> The service attachment name used to create the
    PSC endpoint. AWS_PRIVATELINK -> The service name used to create the VPC
    endpoint.
    """

    direct_route_type: Optional[
        Literal["DIRECT_ROUTE_TYPE_UNSPECIFIED", "INTERNET", "GCP_PRIVATE_SERVICE_CONNECT", "AWS_PRIVATELINK"]
    ] = FieldInfo(alias="directRouteType", default=None)
    """
    If set, this deployment will expose an endpoint that bypasses the Fireworks API
    gateway.
    """

    disable_deployment_size_validation: Optional[bool] = FieldInfo(
        alias="disableDeploymentSizeValidation", default=None
    )
    """Whether the deployment size validation is disabled."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Human-readable display name of the deployment.

    e.g. "My Deployment" Must be fewer than 64 characters long.
    """

    draft_model: Optional[str] = FieldInfo(alias="draftModel", default=None)
    """The draft model name for speculative decoding.

    e.g. accounts/fireworks/models/my-draft-model If empty, speculative decoding
    using a draft model is disabled. Default is the base model's
    default_draft_model. Set CreateDeploymentRequest.disable_speculative_decoding to
    false to disable this behavior.
    """

    draft_token_count: Optional[int] = FieldInfo(alias="draftTokenCount", default=None)
    """
    The number of candidate tokens to generate per step for speculative decoding.
    Default is the base model's draft_token_count. Set
    CreateDeploymentRequest.disable_speculative_decoding to false to disable this
    behavior.
    """

    enable_addons: Optional[bool] = FieldInfo(alias="enableAddons", default=None)
    """If true, PEFT addons are enabled for this deployment."""

    enable_hot_load: Optional[bool] = FieldInfo(alias="enableHotLoad", default=None)
    """Whether to use hot load for this deployment."""

    enable_hot_reload_latest_addon: Optional[bool] = FieldInfo(alias="enableHotReloadLatestAddon", default=None)
    """
    Allows up to 1 addon at a time to be loaded, and will merge it into the base
    model.
    """

    enable_mtp: Optional[bool] = FieldInfo(alias="enableMtp", default=None)
    """If true, MTP is enabled for this deployment."""

    enable_session_affinity: Optional[bool] = FieldInfo(alias="enableSessionAffinity", default=None)
    """Whether to apply sticky routing based on `user` field."""

    expire_time: Optional[datetime] = FieldInfo(alias="expireTime", default=None)
    """The time at which this deployment will automatically be deleted."""

    hot_load_bucket_type: Optional[Literal["BUCKET_TYPE_UNSPECIFIED", "MINIO", "S3", "NEBIUS"]] = FieldInfo(
        alias="hotLoadBucketType", default=None
    )

    max_replica_count: Optional[int] = FieldInfo(alias="maxReplicaCount", default=None)
    """
    The maximum number of replicas. If not specified, the default is
    max(min_replica_count, 1). May be set to 0 to downscale the deployment to 0.
    """

    min_replica_count: Optional[int] = FieldInfo(alias="minReplicaCount", default=None)
    """The minimum number of replicas. If not specified, the default is 0."""

    name: Optional[str] = None

    ngram_speculation_length: Optional[int] = FieldInfo(alias="ngramSpeculationLength", default=None)
    """The length of previous input sequence to be considered for N-gram speculation."""

    num_peft_device_cached: Optional[int] = FieldInfo(alias="numPeftDeviceCached", default=None)

    placement: Optional[Placement] = None
    """
    The desired geographic region where the deployment must be placed. If
    unspecified, the default is the GLOBAL multi-region.
    """

    precision: Optional[
        Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
    ] = None
    """The precision with which the model should be served."""

    purge_time: Optional[datetime] = FieldInfo(alias="purgeTime", default=None)
    """The time at which the resource will be hard deleted."""

    region: Optional[
        Literal[
            "REGION_UNSPECIFIED",
            "US_IOWA_1",
            "US_VIRGINIA_1",
            "US_ILLINOIS_1",
            "AP_TOKYO_1",
            "US_ARIZONA_1",
            "US_TEXAS_1",
            "US_ILLINOIS_2",
            "EU_FRANKFURT_1",
            "US_TEXAS_2",
            "EU_ICELAND_1",
            "EU_ICELAND_2",
            "US_WASHINGTON_1",
            "US_WASHINGTON_2",
            "US_WASHINGTON_3",
            "AP_TOKYO_2",
            "US_CALIFORNIA_1",
            "US_UTAH_1",
            "US_TEXAS_3",
            "US_GEORGIA_1",
            "US_GEORGIA_2",
            "US_WASHINGTON_4",
            "US_GEORGIA_3",
        ]
    ] = None
    """The geographic region where the deployment is presently located.

    This region may change over time, but within the `placement` constraint.
    """

    replica_count: Optional[int] = FieldInfo(alias="replicaCount", default=None)

    state: Optional[Literal["STATE_UNSPECIFIED", "CREATING", "READY", "DELETING", "FAILED", "UPDATING", "DELETED"]] = (
        None
    )
    """The state of the deployment."""

    status: Optional[Status] = None
    """Detailed status information regarding the most recent operation."""

    target_model_version: Optional[str] = FieldInfo(alias="targetModelVersion", default=None)
    """
    The target model version that is being rolled out to the deployment. In a ready
    steady state, the target model version is the same as the active model version.
    """

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the deployment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SecretListParams"]


class SecretListParams(TypedDict, total=False):
    filter: str
    """Unused but required to use existing ListRequest functionality."""

    order_by: Annotated[str, PropertyInfo(alias="orderBy")]
    """Unused but required to use existing ListRequest functionality."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]

    page_token: Annotated[str, PropertyInfo(alias="pageToken")]

    read_mask: Annotated[str, PropertyInfo(alias="readMask")]
    """The fields to be returned in the response.

    If empty or "\\**", all fields will be returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["APIKeyDeleteParams"]


class APIKeyDeleteParams(TypedDict, total=False):
    account_id: Required[str]

    key_id: Required[Annotated[str, PropertyInfo(alias="keyId")]]
    """The key ID for the API key."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.wandb_config import WandbConfig
from .shared_params.training_config import TrainingConfig
from .shared_params.inference_parameters import InferenceParameters

__all__ = ["ReinforcementFineTuningJobCreateParams"]


class ReinforcementFineTuningJobCreateParams(TypedDict, total=False):
    dataset: Required[str]
    """The name of the dataset used for training."""

    evaluator: Required[str]
    """The evaluator resource name to use for RLOR fine-tuning job."""

    reinforcement_fine_tuning_job_id: Annotated[str, PropertyInfo(alias="reinforcementFineTuningJobId")]
    """
    ID of the reinforcement fine-tuning job, a random UUID will be generated if not
    specified.
    """

    chunk_size: Annotated[int, PropertyInfo(alias="chunkSize")]
    """Data chunking for rollout, default size 200, enabled when dataset > 300.

    Valid range is 1-10,000.
    """

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    eval_auto_carveout: Annotated[bool, PropertyInfo(alias="evalAutoCarveout")]
    """Whether to auto-carve the dataset for eval."""

    evaluation_dataset: Annotated[str, PropertyInfo(alias="evaluationDataset")]
    """The name of a separate dataset to use for evaluation."""

    inference_parameters: Annotated[InferenceParameters, PropertyInfo(alias="inferenceParameters")]
    """BIJ parameters."""

    mcp_server: Annotated[str, PropertyInfo(alias="mcpServer")]

    training_config: Annotated[TrainingConfig, PropertyInfo(alias="trainingConfig")]
    """Common training configurations."""

    wandb_config: Annotated[WandbConfig, PropertyInfo(alias="wandbConfig")]
    """The Weights & Biases team/user account for logging training progress."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["WandbConfig"]


class WandbConfig(BaseModel):
    api_key: Optional[str] = FieldInfo(alias="apiKey", default=None)
    """The API key for the wandb service."""

    enabled: Optional[bool] = None
    """Whether to enable wandb logging."""

    entity: Optional[str] = None
    """The entity name for the wandb service."""

    project: Optional[str] = None
    """The project name for the wandb service."""

    run_id: Optional[str] = FieldInfo(alias="runId", default=None)
    """The run ID for the wandb service."""

    url: Optional[str] = None
    """The URL for the wandb service."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ModelPrepareParams"]


class ModelPrepareParams(TypedDict, total=False):
    account_id: Required[str]

    precision: Literal[
        "PRECISION_UNSPECIFIED",
        "FP16",
        "FP8",
        "FP8_MM",
        "FP8_AR",
        "FP8_MM_KV_ATTN",
        "FP8_KV",
        "FP8_MM_V2",
        "FP8_V2",
        "FP8_MM_KV_ATTN_V2",
        "NF4",
        "FP4",
        "BF16",
        "FP4_BLOCKSCALED_MM",
        "FP4_MX_MOE",
    ]

    read_mask: Annotated[str, PropertyInfo(alias="readMask")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "Choice",
    "Logprobs",
    "LogprobsLogProbs",
    "LogprobsNewLogProbs",
    "LogprobsNewLogProbsContent",
    "LogprobsNewLogProbsContentTopLogprob",
    "RawOutput",
    "RawOutputCompletionLogprobs",
    "RawOutputCompletionLogprobsContent",
    "RawOutputCompletionLogprobsContentTopLogprob",
]


class LogprobsLogProbs(BaseModel):
    text_offset: Optional[List[int]] = None

    token_ids: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None


class LogprobsNewLogProbsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class LogprobsNewLogProbsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[LogprobsNewLogProbsContentTopLogprob]] = None


class LogprobsNewLogProbs(BaseModel):
    content: Optional[List[LogprobsNewLogProbsContent]] = None


Logprobs: TypeAlias = Union[LogprobsLogProbs, LogprobsNewLogProbs, None]


class RawOutputCompletionLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class RawOutputCompletionLogprobsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[RawOutputCompletionLogprobsContentTopLogprob]] = None


class RawOutputCompletionLogprobs(BaseModel):
    content: Optional[List[RawOutputCompletionLogprobsContent]] = None


class RawOutput(BaseModel):
    completion: str
    """Raw completion produced by the model before any tool calls are parsed"""

    prompt_fragments: List[Union[str, int]]
    """
    Pieces of the prompt (like individual messages) before truncation and
    concatenation. Depending on prompt_truncate_len some of the messages might be
    dropped. Contains a mix of strings to be tokenized and individual tokens (if
    dictated by the conversation template)
    """

    prompt_token_ids: List[int]
    """Fully processed prompt as seen by the model"""

    completion_logprobs: Optional[RawOutputCompletionLogprobs] = None
    """OpenAI-compatible log probabilities format"""

    completion_token_ids: Optional[List[int]] = None
    """Token IDs for the raw completion"""


class Choice(BaseModel):
    index: int
    """The index of the completion choice"""

    text: str
    """The completion response"""

    finish_reason: Optional[Literal["stop", "length", "error"]] = None
    """The reason the model stopped generating tokens.

    This will be "stop" if the model hit a natural stop point or a provided stop
    sequence, or "length" if the maximum number of tokens specified in the request
    was reached
    """

    logprobs: Optional[Logprobs] = None
    """The log probabilities of the most likely tokens"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true)"""

    raw_output: Optional[RawOutput] = None
    """
    Extension of OpenAI that returns low-level interaction of what the model sees,
    including the formatted prompt and function calls
    """

    token_ids: Optional[List[int]] = None
    """Token IDs for the generated completion (when return_token_ids=true)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .splitted import Splitted
from .transformed import Transformed
from .shared.status import Status
from .evaluation_result import EvaluationResult

__all__ = ["Dataset"]


class Dataset(BaseModel):
    average_turn_count: Optional[float] = FieldInfo(alias="averageTurnCount", default=None)
    """An estimate of the average number of turns per sample in the dataset."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The email address of the user who initiated this fine-tuning job."""

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)

    estimated_token_count: Optional[str] = FieldInfo(alias="estimatedTokenCount", default=None)
    """The estimated number of tokens in the dataset."""

    eval_protocol: Optional[object] = FieldInfo(alias="evalProtocol", default=None)

    evaluation_result: Optional[EvaluationResult] = FieldInfo(alias="evaluationResult", default=None)

    example_count: Optional[str] = FieldInfo(alias="exampleCount", default=None)

    external_url: Optional[str] = FieldInfo(alias="externalUrl", default=None)

    format: Optional[Literal["FORMAT_UNSPECIFIED", "CHAT", "COMPLETION", "RL"]] = None

    name: Optional[str] = None

    source_job_name: Optional[str] = FieldInfo(alias="sourceJobName", default=None)
    """
    The resource name of the job that created this dataset (e.g., batch inference
    job). Used for lineage tracking to understand dataset provenance.
    """

    splitted: Optional[Splitted] = None

    state: Optional[Literal["STATE_UNSPECIFIED", "UPLOADING", "READY"]] = None

    status: Optional[Status] = None

    transformed: Optional[Transformed] = None

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the dataset."""

    user_uploaded: Optional[object] = FieldInfo(alias="userUploaded", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["WandbConfig"]


class WandbConfig(TypedDict, total=False):
    api_key: Annotated[str, PropertyInfo(alias="apiKey")]
    """The API key for the wandb service."""

    enabled: bool
    """Whether to enable wandb logging."""

    entity: str
    """The entity name for the wandb service."""

    project: str
    """The project name for the wandb service."""

    run_id: Annotated[str, PropertyInfo(alias="runId")]
    """The run ID for the wandb service."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .reinforcement_fine_tuning_job import ReinforcementFineTuningJob

__all__ = ["ReinforcementFineTuningJobListResponse"]


class ReinforcementFineTuningJobListResponse(BaseModel):
    next_page_token: Optional[str] = FieldInfo(alias="nextPageToken", default=None)
    """
    A token, which can be sent as `page_token` to retrieve the next page. If this
    field is omitted, there are no subsequent pages.
    """

    reinforcement_fine_tuning_jobs: Optional[List[ReinforcementFineTuningJob]] = FieldInfo(
        alias="reinforcementFineTuningJobs", default=None
    )

    total_size: Optional[int] = FieldInfo(alias="totalSize", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from ..shared.chat_completion_message_tool_call import ChatCompletionMessageToolCall

__all__ = [
    "ChatCompletionChunk",
    "Choice",
    "ChoiceDelta",
    "ChoiceLogprobs",
    "ChoiceLogprobsLogProbs",
    "ChoiceLogprobsNewLogProbs",
    "ChoiceLogprobsNewLogProbsContent",
    "ChoiceLogprobsNewLogProbsContentTopLogprob",
    "ChoiceRawOutput",
    "ChoiceRawOutputCompletionLogprobs",
    "ChoiceRawOutputCompletionLogprobsContent",
    "ChoiceRawOutputCompletionLogprobsContentTopLogprob",
    "Usage",
    "UsagePromptTokensDetails",
]


class ChoiceDelta(BaseModel):
    content: Optional[str] = None
    """The contents of the chunk message"""

    reasoning_content: Optional[str] = None
    """The reasoning or thinking process generated by the model.

    This field is only available for certain reasoning models (GLM 4.5, GLM 4.5 Air,
    GPT OSS 120B, GPT OSS 20B) and contains the model's internal reasoning that
    would otherwise appear in `<think></think>` tags within the content field.
    """

    role: Optional[str] = None
    """The role of the author of this message"""

    tool_calls: Optional[List[ChatCompletionMessageToolCall]] = None


class ChoiceLogprobsLogProbs(BaseModel):
    text_offset: Optional[List[int]] = None

    token_ids: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None


class ChoiceLogprobsNewLogProbsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class ChoiceLogprobsNewLogProbsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[ChoiceLogprobsNewLogProbsContentTopLogprob]] = None


class ChoiceLogprobsNewLogProbs(BaseModel):
    content: Optional[List[ChoiceLogprobsNewLogProbsContent]] = None


ChoiceLogprobs: TypeAlias = Union[ChoiceLogprobsLogProbs, ChoiceLogprobsNewLogProbs, None]


class ChoiceRawOutputCompletionLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    token_id: int

    bytes: Optional[List[int]] = None


class ChoiceRawOutputCompletionLogprobsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[ChoiceRawOutputCompletionLogprobsContentTopLogprob]] = None


class ChoiceRawOutputCompletionLogprobs(BaseModel):
    content: Optional[List[ChoiceRawOutputCompletionLogprobsContent]] = None


class ChoiceRawOutput(BaseModel):
    completion: str
    """Raw completion produced by the model before any tool calls are parsed"""

    prompt_fragments: List[Union[str, int]]
    """
    Pieces of the prompt (like individual messages) before truncation and
    concatenation. Depending on prompt_truncate_len some of the messages might be
    dropped. Contains a mix of strings to be tokenized and individual tokens (if
    dictated by the conversation template)
    """

    prompt_token_ids: List[int]
    """Fully processed prompt as seen by the model"""

    completion_logprobs: Optional[ChoiceRawOutputCompletionLogprobs] = None
    """OpenAI-compatible log probabilities format"""

    completion_token_ids: Optional[List[int]] = None
    """Token IDs for the raw completion"""


class Choice(BaseModel):
    delta: ChoiceDelta
    """The message delta"""

    index: int
    """The index of the chat completion choice"""

    finish_reason: Optional[Literal["stop", "length", "function_call", "tool_calls"]] = None
    """The reason the model stopped generating tokens.

    This will be "stop" if the model hit a natural stop point or a provided stop
    sequence, or "length" if the maximum number of tokens specified in the request
    was reached
    """

    logprobs: Optional[ChoiceLogprobs] = None
    """Legacy log probabilities format"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true)"""

    raw_output: Optional[ChoiceRawOutput] = None
    """
    Extension of OpenAI that returns low-level interaction of what the model sees,
    including the formatted prompt and function calls
    """

    token_ids: Optional[List[int]] = None
    """Token IDs for this chunk (when return_token_ids=true)"""


class UsagePromptTokensDetails(BaseModel):
    cached_tokens: Optional[int] = None


class Usage(BaseModel):
    prompt_tokens: int
    """The number of tokens in the prompt"""

    total_tokens: int
    """The total number of tokens used in the request (prompt + completion)"""

    completion_tokens: Optional[int] = None
    """The number of tokens in the generated completion"""

    prompt_tokens_details: Optional[UsagePromptTokensDetails] = None
    """Details about prompt tokens, including cached tokens"""


class ChatCompletionChunk(BaseModel):
    id: str
    """A unique identifier of the response"""

    choices: List[Choice]
    """The list of streamed chat completion choices"""

    created: int
    """The Unix time in seconds when the response was generated"""

    model: str
    """The model used for the chat completion"""

    object: Optional[str] = None
    """The object type, which is always "chat.completion.chunk" """

    perf_metrics: Optional[Dict[str, builtins.object]] = None
    """See parameter [perf_metrics_in_response](#body-perf-metrics-in-response)"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true, sent in first chunk)"""

    usage: Optional[Usage] = None
    """Usage statistics."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .dataset_param import DatasetParam

__all__ = ["DatasetCreateParams"]


class DatasetCreateParams(TypedDict, total=False):
    dataset: Required[DatasetParam]

    dataset_id: Required[Annotated[str, PropertyInfo(alias="datasetId")]]

    filter: str

    source_dataset_id: Annotated[str, PropertyInfo(alias="sourceDatasetId")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .status import Status
from ..._models import BaseModel

__all__ = ["DeployedModel"]


class DeployedModel(BaseModel):
    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the resource."""

    default: Optional[bool] = None
    """
    If true, this is the default target when querying this model without the
    `#<deployment>` suffix. The first deployment a model is deployed to will have
    this field set to true.
    """

    deployment: Optional[str] = None
    """The resource name of the base deployment the model is deployed to."""

    description: Optional[str] = None
    """Description of the resource."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)

    model: Optional[str] = None

    name: Optional[str] = None

    public: Optional[bool] = None
    """If true, the deployed model will be publicly reachable."""

    serverless: Optional[bool] = None

    state: Optional[Literal["STATE_UNSPECIFIED", "UNDEPLOYING", "DEPLOYING", "DEPLOYED", "UPDATING"]] = None
    """The state of the deployed model."""

    status: Optional[Status] = None
    """Contains model deploy/undeploy details."""

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the deployed model."""
